#include "MainWindow.h"
#include "spacer.h"
#include "Constants.h"
#include "iconloader/qticonloader.h"
#include "global.h"

MainWindow::MainWindow() {

    m_fullscreen = false;

    // Phonon initialization
    mediaObject = new Phonon::MediaObject(this);
    connect(mediaObject, SIGNAL(stateChanged(Phonon::State, Phonon::State)),
            this, SLOT(stateChanged(Phonon::State, Phonon::State)));
    audioOutput = new Phonon::AudioOutput(Phonon::VideoCategory, this);
    Phonon::createPath(mediaObject, audioOutput);

    // views mechanism
    history = new QStack<QWidget*>();
    views = new QStackedWidget(this);

    // views
    searchView = new SearchView(this);
    connect(searchView, SIGNAL(search(QString)), this, SLOT(showMedia(QString)));
    views->addWidget(searchView);
    mediaView = new MediaView(this);
    mediaView->setMediaObject(mediaObject);
    views->addWidget(mediaView);

    // lazy initialized view
    aboutView = 0;
    settingsView = 0;

    toolbarSearch = new SearchLineEdit(this);
    toolbarSearch->setFont(qApp->font());
    // toolbarSearch->setMinimumWidth(200);
    connect(toolbarSearch, SIGNAL(search(const QString&)), searchView, SLOT(watch(const QString&)));

    // build ui
    createActions();
    createMenus();
    createToolBars();
    createStatusBar();
    readSettings();

    // show the initial view
    showWidget(searchView);

    setCentralWidget(views);

    // top dock widget
    /*
    QLabel* message = new QLabel(this);
    message->setText(QString("A new version of %1 is available.").arg(Constants::APP_NAME));
    message->setMargin(10);
    message->setAlignment(Qt::AlignCenter);
    QPalette palette;
    message->setBackgroundRole(QPalette::ToolTipBase);
    message->setForegroundRole(QPalette::ToolTipText);
    message->setAutoFillBackground(true);
    QDockWidget *dockWidget = new QDockWidget("", this, 0);
    dockWidget->setTitleBarWidget(0);
    dockWidget->setWidget(message);
    dockWidget->setFeatures(QDockWidget::DockWidgetClosable);
    addDockWidget(Qt::TopDockWidgetArea, dockWidget);
    */

}

MainWindow::~MainWindow() {
    delete history;
}

void MainWindow::createActions() {

    QMap<QString, QAction*> *actions = The::globalActions();

    settingsAct = new QAction(tr("&Preferences..."), this);
    settingsAct->setStatusTip(tr(QString("Configure ").append(Constants::APP_NAME).toUtf8()));
    // Mac integration
    settingsAct->setMenuRole(QAction::PreferencesRole);
    actions->insert("settings", settingsAct);
    connect(settingsAct, SIGNAL(triggered()), this, SLOT(showSettings()));
    
    backAct = new QAction(QIcon(":/images/go-previous.png"), tr("&Back"), this);
    backAct->setEnabled(false);
    backAct->setShortcut(tr("Alt+Left"));
    backAct->setStatusTip(tr("Go to the previous view"));
    actions->insert("back", backAct);
    connect(backAct, SIGNAL(triggered()), this, SLOT(goBack()));

    stopAct = new QAction(QtIconLoader::icon("media-stop", QIcon(":/images/stop.png")), tr("&Stop"), this);
    stopAct->setStatusTip(tr("Stop playback and go back to the search view"));
    stopAct->setToolTip(stopAct->statusTip());
    stopAct->setShortcut(tr("Esc"));
    // stopAct->setEnabled(false);
    actions->insert("stop", stopAct);
    connect(stopAct, SIGNAL(triggered()), this, SLOT(stop()));

    skipAct = new QAction(QtIconLoader::icon("media-skip-forward", QIcon(":/images/skip.png")), tr("S&kip"), this);
    skipAct->setStatusTip(tr("Skip this video"));
    skipAct->setShortcut(tr("Ctrl+Right"));
    skipAct->setEnabled(false);
    actions->insert("skip", skipAct);
    connect(skipAct, SIGNAL(triggered()), mediaView, SLOT(skip()));

    pauseAct = new QAction(QtIconLoader::icon("media-pause", QIcon(":/images/pause.png")), tr("&Pause"), this);
    pauseAct->setStatusTip(tr("Pause playback"));
    pauseAct->setShortcut(tr("Space"));
    pauseAct->setEnabled(false);
    actions->insert("pause", pauseAct);
    connect(pauseAct, SIGNAL(triggered()), mediaView, SLOT(pause()));

    fullscreenAct = new QAction(QtIconLoader::icon("view-fullscreen", QIcon(":/images/view-fullscreen.png")), tr("&Full Screen"), this);
    fullscreenAct->setStatusTip(tr("Go full screen"));
    fullscreenAct->setShortcut(tr("Alt+Return"));
    fullscreenAct->setShortcutContext(Qt::ApplicationShortcut);
    actions->insert("fullscreen", fullscreenAct);
    connect(fullscreenAct, SIGNAL(triggered()), this, SLOT(fullscreen()));

    /*
    // icon should be document-save but it is ugly
    downloadAct = new QAction(QtIconLoader::icon("go-down", QIcon(":/images/go-down.png")), tr("&Download"), this);
    downloadAct->setStatusTip(tr("Download this video"));
    downloadAct->setShortcut(tr("Ctrl+S"));
    actions.insert("download", downloadAct);
    connect(downloadAct, SIGNAL(triggered()), this, SLOT(download()));
    */

    webPageAct = new QAction(QtIconLoader::icon("internet-web-browser", QIcon(":/images/internet-web-browser.png")), tr("&YouTube"), this);
    webPageAct->setStatusTip(tr("Open the YouTube video page"));
    webPageAct->setShortcut(tr("Ctrl+Y"));
    webPageAct->setEnabled(false);
    actions->insert("webpage", webPageAct);
    connect(webPageAct, SIGNAL(triggered()), mediaView, SLOT(openWebPage()));

    removeAct = new QAction(tr("&Remove"), this);
    removeAct->setStatusTip(tr("Remove the selected videos from the playlist"));
    removeAct->setShortcut(tr("Del"));
    removeAct->setEnabled(false);
    actions->insert("remove", removeAct);
    connect(removeAct, SIGNAL(triggered()), mediaView, SLOT(removeSelected()));

    moveUpAct = new QAction(QtIconLoader::icon("go-up", QIcon(":/images/go-up.png")), tr("Move &Up"), this);
    moveUpAct->setStatusTip(tr("Move up the selected videos in the playlist"));
    moveUpAct->setShortcut(tr("Ctrl+Up"));
    moveUpAct->setEnabled(false);
    actions->insert("moveUp", moveUpAct);
    connect(moveUpAct, SIGNAL(triggered()), mediaView, SLOT(moveUpSelected()));

    moveDownAct = new QAction(QtIconLoader::icon("go-down", QIcon(":/images/go-down.png")), tr("Move &Down"), this);
    moveDownAct->setStatusTip(tr("Move down the selected videos in the playlist"));
    moveDownAct->setShortcut(tr("Ctrl+Down"));
    moveDownAct->setEnabled(false);
    actions->insert("moveDown", moveDownAct);
    connect(moveDownAct, SIGNAL(triggered()), mediaView, SLOT(moveDownSelected()));

    quitAct = new QAction(tr("&Quit"), this);
    quitAct->setShortcut(tr("Ctrl+Q"));
    quitAct->setStatusTip(tr("Bye bye"));
    actions->insert("quit", quitAct);
    connect(quitAct, SIGNAL(triggered()), this, SLOT(quit()));

    siteAct = new QAction(tr("&Website"), this);
    siteAct->setShortcut(tr("F1"));
    siteAct->setStatusTip(tr("Minitube on the Web"));
    actions->insert("site", siteAct);
    connect(siteAct, SIGNAL(triggered()), this, SLOT(visitSite()));

    donateAct = new QAction(tr("&Donate via PayPal"), this);
    donateAct->setStatusTip(tr("Please support the continued development of %1").arg(Constants::APP_NAME));
    actions->insert("donate", donateAct);
    connect(donateAct, SIGNAL(triggered()), this, SLOT(donate()));

    aboutAct = new QAction(tr("&About"), this);
    aboutAct->setStatusTip(tr("Info about %1").arg(Constants::APP_NAME));
    actions->insert("about", aboutAct);
    connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

    searchAct = new QAction(tr("Search"), this);
    searchAct->setShortcut(tr("Ctrl+F"));
    actions->insert("search", searchAct);
    connect(searchAct, SIGNAL(triggered()), toolbarSearch, SLOT(setFocus()));

    // common action properties
    foreach (QAction *action, actions->values()) {
        // never autorepeat.
        // unexperienced users tend to keep keys pressed for a "long" time
        action->setAutoRepeat(false);
    }


}

void MainWindow::createMenus() {
    fileMenu = menuBar()->addMenu(tr("&Application"));
    /*
    fileMenu->addAction(settingsAct);
    fileMenu->addSeparator();
    */
    fileMenu->addAction(quitAct);

    playlistMenu = menuBar()->addMenu(tr("&Playlist"));
    playlistMenu->addAction(removeAct);
    playlistMenu->addSeparator();
    playlistMenu->addAction(moveUpAct);
    playlistMenu->addAction(moveDownAct);

    viewMenu = menuBar()->addMenu(tr("&Video"));
    // viewMenu->addAction(backAct);
    viewMenu->addAction(stopAct);
    viewMenu->addAction(pauseAct);
    viewMenu->addAction(skipAct);
    viewMenu->addSeparator();
    viewMenu->addAction(fullscreenAct);
    viewMenu->addSeparator();
    viewMenu->addAction(webPageAct);

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(siteAct);
    helpMenu->addAction(donateAct);
    helpMenu->addAction(aboutAct);
}

void MainWindow::createToolBars() {

    // This is cool on the Mac
    // But does not respect layouts, maybe it's a Qt bug
    // setUnifiedTitleAndToolBarOnMac(true);

    setToolButtonStyle(Qt::ToolButtonTextUnderIcon);

    mainToolBar = new QToolBar("This is a useless menu that the Qt toolkit won't let me remove", this);
    mainToolBar->setFloatable(false);
    mainToolBar->setMovable(false);

    QFont smallerFont;
    smallerFont.setPointSize(smallerFont.pointSize()*.85);
    mainToolBar->setFont(smallerFont);

    mainToolBar->setIconSize(QSize(32,32));
    // mainToolBar->addAction(backAct);
    mainToolBar->addAction(stopAct);
    mainToolBar->addAction(pauseAct);
    mainToolBar->addAction(skipAct);
    mainToolBar->addAction(fullscreenAct);

    seekSlider = new Phonon::SeekSlider(this);
    seekSlider->setMediaObject(mediaObject);
    seekSlider->setIconVisible(false);
    mainToolBar->addWidget(new Spacer(mainToolBar, seekSlider));

    volumeSlider = new Phonon::VolumeSlider(this);
    volumeSlider->setAudioOutput(audioOutput);
    // this makes the volume slider smaller...
    volumeSlider->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
    mainToolBar->addWidget(new Spacer(mainToolBar, volumeSlider));

    // mainToolBar->addSeparator();
    // mainToolBar->addAction(downloadAct);
    // mainToolBar->addAction(webPageAct);

    mainToolBar->addWidget(new Spacer(mainToolBar, toolbarSearch));

    addToolBar(mainToolBar);
}

void MainWindow::createStatusBar() {
    statusBar()->show();
}

void MainWindow::readSettings() {
    QSettings settings;
    restoreGeometry(settings.value("geometry").toByteArray());
}

void MainWindow::writeSettings() {
    QSettings settings;
    settings.setValue("geometry", saveGeometry());
}

void MainWindow::goBack() {
    if ( history->size() > 1 ) {
        history->pop();
        QWidget *widget = history->pop();
        showWidget(widget);
    }
}

void MainWindow::showWidget ( QWidget* widget ) {

    // call hide method on the current view
    View* oldView = dynamic_cast<View *> (views->currentWidget());
    if (oldView != NULL) {
        oldView->disappear();
    }

    // call show method on the new view
    View* newView = dynamic_cast<View *> (widget);
    if (newView != NULL) {
        newView->appear();
        QMap<QString,QVariant> metadata = newView->metadata();
        QString windowTitle = metadata.value("title").toString();
        if (windowTitle.length())
            windowTitle += " - ";
        setWindowTitle(windowTitle + Constants::APP_NAME);
        setStatusTip(metadata.value("description").toString());
    }

    // backAct->setEnabled(history->size() > 1);
    stopAct->setEnabled(widget == mediaView);
    fullscreenAct->setEnabled(widget == mediaView);
    webPageAct->setEnabled(widget == mediaView);
    settingsAct->setEnabled(widget != settingsView);
    aboutAct->setEnabled(widget != aboutView);
    // toolbar only for the mediaView
    mainToolBar->setVisible(widget == mediaView);

    history->push(widget);
    fadeInWidget(views->currentWidget(), widget);
    views->setCurrentWidget(widget);

}

void MainWindow::fadeInWidget(QWidget *oldWidget, QWidget *newWidget) {
    if (faderWidget) faderWidget->close();
    if (!oldWidget || !newWidget || oldWidget == mediaView || newWidget == mediaView) return;
    QPixmap frozenView = QPixmap::grabWidget(oldWidget);
    faderWidget = new FaderWidget(newWidget);
    faderWidget->start(frozenView);
}

void MainWindow::about() {
    if (!aboutView) {
        aboutView = new AboutView(this);
        views->addWidget(aboutView);
    }
    showWidget(aboutView);
}

void MainWindow::visitSite() {
    QUrl url(Constants::WEBSITE);
    statusBar()->showMessage(QString(tr("Opening %1").arg(url.toString())));
    QDesktopServices::openUrl(url);
}

void MainWindow::donate() {
    QUrl url(QString(Constants::WEBSITE) + "#donate");
    statusBar()->showMessage(QString(tr("Opening %1").arg(url.toString())));
    QDesktopServices::openUrl(url);
}

void MainWindow::quit() {
    writeSettings();
    qApp->quit();
}

void MainWindow::showSettings() {
    if (!settingsView) {
        settingsView = new SettingsView(this);
        views->addWidget(settingsView);
    }
    showWidget(settingsView);
}

void MainWindow::showSearch() {
    showWidget(searchView);
}

void MainWindow::showMedia(QString query) {
    mediaView->setQuery(query);
    showWidget(mediaView);
}

void MainWindow::stateChanged(Phonon::State newState, Phonon::State /* oldState */) {

    // qDebug() << "Phonon state: " << newState;

    switch (newState) {

         case Phonon::ErrorState:
        if (mediaObject->errorType() == Phonon::FatalError) {
            setStatusTip("Something went REALLY wrong: " + mediaObject->errorString());
        } else {
            setStatusTip("Something went wrong: " + mediaObject->errorString());
        }
        break;

         case Phonon::PlayingState:
        pauseAct->setEnabled(true);
        pauseAct->setIcon(QtIconLoader::icon("media-pause", QIcon(":/images/pause.png")));
        pauseAct->setText(tr("&Pause"));
        pauseAct->setStatusTip(tr("Pause playback"));
        // stopAct->setEnabled(true);
        skipAct->setEnabled(true);
        break;

         case Phonon::StoppedState:
        // stopAct->setEnabled(false);
        pauseAct->setEnabled(false);
        skipAct->setEnabled(false);
        break;

         case Phonon::PausedState:
        // stopAct->setEnabled(true);
        skipAct->setEnabled(true);
        pauseAct->setEnabled(true);
        pauseAct->setIcon(QtIconLoader::icon("media-play", QIcon(":/images/play.png")));
        pauseAct->setText(tr("&Play"));
        pauseAct->setStatusTip(tr("Resume playback"));
        break;

         case Phonon::BufferingState:
         case Phonon::LoadingState:
        // stopAct->setEnabled(true);
        skipAct->setEnabled(true);
        pauseAct->setEnabled(false);
        break;

         default:
        ;
    }
}

void MainWindow::stop() {
    mediaView->stop();
    showSearch();
}

void MainWindow::fullscreen() {
    if (m_fullscreen) {
        mediaView->exitFullscreen();
        fullscreenAct->setShortcut(tr("Alt+Return"));
        stopAct->setShortcut(tr("Esc"));
    } else {
        mediaView->fullscreen();
        stopAct->setShortcut(tr(""));
        fullscreenAct->setShortcut(tr("Esc"));
    }
    m_fullscreen = !m_fullscreen;
    /*
    mainToolBar->setVisible(full && views->currentWidget() == mediaView);
    menuBar()->setVisible(full);
    statusBar()->setVisible(full);
    */
}
